module SiteMandala

  class SiteMandalaError < StandardError
    attr_reader :data
    def initialize(data)
      @data = data
      extra = @data.ResultData ? "\nExtra result data: #{@data.ResultData}" : ""
      super "The SiteMandala API responded with the following error - #{@data.Code}: #{@data.Message}#{extra}"
    end
  end

  class ClientError < StandardError; end
  class ServerError < StandardError; end
  class BadRequest < SiteMandalaError; end
  class Unauthorized < SiteMandalaError; end
  class NotFound < ClientError; end
  class InvalidOAuthToken < Unauthorized; end
  class ExpiredOAuthToken < Unauthorized; end
  class RevokedOAuthToken < Unauthorized; end

  class SiteMandala
    
    class << self
      
      attr_accessor :token
      
    end
    
    def self.get(endpoint)
      url = generate_url(endpoint)
      handle_response HTTParty.get(url, :format => :json)
    end
  
    def self.post(endpoint, data)
      url = generate_url(endpoint)
      handle_response HTTParty.post(url, :body => data.to_json, :format => :json)
    end
  
    def self.put(endpoint, data)
      url = generate_url(endpoint)
      handle_response HTTParty.put(url, :body => data.to_json, :format => :json)
    end
  
    def self.patch(endpoint, data)
      url = generate_url(endpoint)
      handle_response HTTParty.patch(url, :body => data.to_json, :format => :json)
    end
  
    def self.delete(endpoint)
      url = generate_url(endpoint)
      handle_response HTTParty.delete(url, :format => :json)
    end
    
    def self.generate_url(endpoint)
      Configuration.ensure! :base_url
      Configuration.ensure! :token
      token = Configuration.token
      base_url = Configuration.base_url
      base_url+endpoint+"?token="+token
    end
    
    def self.handle_response(response)
      case response.code
      when 400
        raise BadRequest.new(Hashie::Mash.new response)
      when 401
        data = Hashie::Mash.new(response)
      when 404
        raise NotFound.new
      when 400...500
        raise ClientError.new
      when 500...600
        raise ServerError.new
      else
        records = response.parsed_response
        if records.is_a?(Array)
          records.map{ |item| Hashie::Mash.new(item) }
        elsif records.is_a?(Hash)
          Hashie::Mash.new(records)
        end
      end
    end  
  end

end